#ifndef lint
static char	*RCSid = "$Id$";
#endif

/*
 * list.c - routines to handle the list of hidden notes.
 *
 * Michael J. Hammel
 * Contractor
 * 1150 Inca St. TH 70
 * Denver, CO 80204
 * mjhammel@csn.org
 *
 * $Log$
 * 
 */
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Shell.h>
#include <stdio.h>
#include <string.h>

#include "xpostit.h"

/* globals */
Widget		listwidget;		/* the menu of hidden notes */
Widget 		error;		/* error dialog box */

/* local variables */
static ItemList		itemlist[DefaultMaxNotes];
static int				itemcount=0;

/* external variables */
extern Widget			toplevel;
extern Widget			notelist[DefaultMaxNotes];

/* external routines */
extern PostItNote	*FindNote();

static void HandleListSelection();
static void ErrPopDown();

/*
 * CreateListWidget - create the widget used for the list.
 */
void
CreateListWidget()
{
	Arg 				args[8];
	Widget 			listline;
	register int 	nargs;


	/*
	 * Create a popup shell widget
	 */
	nargs = 0;
	SetArg(XtNlabel, PostItNoteList);

	listwidget = XtCreatePopupShell(
						"List", 
						simpleMenuWidgetClass,
						toplevel, 
						args, nargs);

	/*
	 * Create the title bar and line seperator
	 */
	listline = XtCreateManagedWidget(
				"listline",
				smeLineObjectClass,
				listwidget,
				NULL, 0);
}

/*
 * CreateListWidget - create the widget used for the list.
 */
void
CreateErrorDialog()
{
	Arg 				args[3];
	register int 	nargs;
	Widget			errdialog, errconfirm;


	/*
	 * Create a popup shell widget
	 */
	nargs = 0;
	SetArg(XtNlabel, PostItErrDialog);

	error = XtCreatePopupShell(
						"List", 
						transientShellWidgetClass,
						toplevel, 
						args, nargs);

	/*
	 * Create the error dialog box
	 */
	nargs = 0;
	SetArg(XtNlabel, "No more notes can be hidden!");
	errdialog = XtCreateManagedWidget(
				"error",
				dialogWidgetClass,
				error,
				args, nargs);

	/*
	 * Create the error dialog confirmation button
	 */
	nargs = 0;
	SetArg(XtNcallback, ErrPopDown);
	SetArg(XtNlabel, "Continue");
	errconfirm = XtCreateManagedWidget(
				"errconfirm",
				commandWidgetClass,
				errdialog,
				args, nargs);

}

/*
 * AddListItem - add a note to the list of hidden notes
 */
void
AddListItem(
	String		item,
	PostItNote	*pn
)
{
	Arg 				args[3];
	register int 	nargs;
	String 			newitem;
	char				name[10];
	int				i;

	/* add the item to the array of strings */

	/* find the first entry thats not being used */
	if ( itemcount < DefaultMaxNotes )
	{
		for ( i=0; i < DefaultMaxNotes; i++ )
		{
			if ( itemlist[i].name == NULL )
				break;
		}

		if ( i != DefaultMaxNotes )
		{
			newitem = XtNewString ( item );
			itemlist[i].name = newitem;
			notelist[i] = pn->pn_shellwidget;

			sprintf(name, "item%d", itemcount);
			nargs=0;
			SetArg(XtNlabel, newitem);

			itemlist[i].widget = XtCreateManagedWidget(
						name, 
						smeBSBObjectClass, 
						listwidget,
						args, nargs);

			pn->pn_hide_index = i;
			XtAddCallback( itemlist[i].widget, 
					XtNcallback, HandleListSelection, (XtPointer)pn->pn_index);
			itemcount++;
		}
		else
			XtPopup( error, XtGrabNonexclusive );
	}
	else
		XtPopup( error, XtGrabNonexclusive );
}


/*
 * PopUpList - brings up the list of hidden notes
 */
void
PopUpList(
)
{
	XtPopup( listwidget, XtGrabNonexclusive );
}

/*
 * ErrPopDown - removes the Error Dialog box
 */
void
ErrPopDown(
)
{
	XtPopdown( error );
}



/*
 * HandleListSelection - callback for a selected list item
 */
static void
HandleListSelection(
	Widget list,
	caddr_t client_data, 
	caddr_t call_data 
)
{
	int			index;
	PostItNote	*pn;

	/*
	 * Find the note we're erasing.
	 */
	if ((pn = FindNote((int) client_data)) == NULL)
		return;

	index = pn->pn_hide_index;

	/* pop up the note */
	XtPopup( notelist[index], XtGrabNone);
	pn->pn_hidden = False;
	pn->pn_hide_index = -1;

	XtDestroyWidget ( itemlist[index].widget );

	/* remove the item from the list */
	if ( itemlist[index].name != NULL )
		XtFree( itemlist[index].name );
	itemlist[index].name 	= (String)NULL;	/* no String */
	itemlist[index].widget	= (Widget)NULL;	/* not widget */
	notelist[index] 			= (Widget)NULL;	/* no widget */

	/* we've reduced the total count of items */
	if ( itemcount > 0 )
		itemcount--;
	else
		/* this should never happen, either */
		printf ( "xpostit:  just deleted a non-existant hidden note!\n");

}
